% This script takes light output measurements from optical CIs measured with an integrating sphere.
% LEDs of the implants were driven individually at different current intensities, furthermore all LEDs have been driven combined at different current intensities.
% Blockwise LED illumination (of 4 LEDs at a time) is estimated based on the calibration curve of individual LEDs, as this has not been measured directly. 
% The resulting figure displays the mean (+/s standard deviation) radiant flux of the oCI driven with different current intensities.

clear all; close all; clc;

calib_path = 'E:\projects\paper_uLED\revision\data\data_LED_calibration'; % CHANGE TO ACTUAL PATH
files = dir(fullfile(calib_path, '*.mat'));

indiv_leds = [];
block_leds = [];
all_leds = [];
intensity_estimates_x = 0:2.5:25; % percentage of the driving current of individual LEDs at which the intensity is estimated. Multiplied by 4, this results in the estimation of blockwise LED illumination, as 4 LEDs are driven with the same current as individual LEDs, but the current is distributed within the block.

n_leds = 16; 
n_blocks = 4; 

max_current_individual_uLEDs = 10;
max_current_all_uLEDs = 40; 

intensity_conversion = 1/127*100;% converts stimulation intensity( set as 7 bit values (2**7-1)) in percent of maximum intensity

led_power_threshold = 0.0001; % this threshold of 0.1 mW (at maximum driving current) was chosen to identify dysfunctional LEDs and exclude them from calculating the mean 


for i_file = 1:size(files, 1) % for each file in this folder...
    
    cur_file = files(i_file).name; cur_data = load([calib_path '\' cur_file]); % load the data
    driving_current_perc = cur_data.LEDIntensityDevBoard*intensity_conversion; % get driving current and convert it in percent of maximum
    led_intensities = [cur_data.LEDcalUlbricht(:, 2:end) cur_data.LEDcalUlbricht(:, 1)]; % get led intensity, a re-arrangement is necessary because highest intensity is (accidentally) saved at index 1
    
    indiv_leds  = vertcat(indiv_leds, led_intensities(1:n_leds, :)); % get data for individual LEDs
    all_leds    = vertcat(all_leds, led_intensities(end, :)); % get data for simultaneous activation of all LEDs
     
    for i_block = 1:n_blocks % estimate data for simultaneous block activation from summation of individual LEDs
        cur_block = (led_intensities(i_block*n_blocks-3:i_block*n_blocks, :));
        
        intensity_estimates_y = []; 
        
        for i_block_no = 1:size(cur_block, 1)
            cur_block_led = cur_block(i_block_no, :);
            for i_intensities = 1:size(intensity_estimates_x, 2)
                intensity_estimates_y(i_intensities, i_block_no) = interp1(driving_current_perc,cur_block_led,intensity_estimates_x(i_intensities));
            end
        end
        block_leds = vertcat(block_leds, mean(intensity_estimates_y').*n_blocks);
    end
    
end


indiv_leds = fliplr(indiv_leds); all_leds = fliplr(all_leds); driving_current_perc = fliplr(driving_current_perc);

% seperate in functional and unfunctional LEDs/Blocks
functional_leds = indiv_leds( mean(indiv_leds') > led_power_threshold, :); unfunctional_leds = indiv_leds( mean(indiv_leds') < led_power_threshold, :); 
functional_blocks = block_leds(mean(block_leds') > led_power_threshold, :); unfunctional_blocks = block_leds(mean(block_leds') < led_power_threshold, :); 


% plot data
f1 = figure('color', [1 1 1]);

subplot(3, 1, 1); hold on;
x_values_indiv = driving_current_perc./100*max_current_individual_uLEDs;
plot(x_values_indiv, unfunctional_leds, 'k'); plot(x_values_indiv, functional_leds, 'c'); % maximum current used was 10 mA
errorbar(x_values_indiv, mean(functional_leds), std(functional_leds), 'b', 'LineWidth', 2);
box on; xlabel('current [mA]'); ylabel('power [mW]'); title('individual LEDs'); grid on;
xlim([min(x_values_indiv) max(x_values_indiv)]); ylim([0 2/1000])
text(0.5, 0.00175, ['n = ' num2str(size(functional_leds, 1)) '/' num2str(size(indiv_leds, 1)) ' LEDs'])

subplot(3, 1, 2); hold on;
x_values_block = intensity_estimates_x/100*max_current_individual_uLEDs*n_blocks;
plot(x_values_block, unfunctional_blocks, 'k'); plot(x_values_block, functional_blocks, 'c');  % x-axis: maximum current used was 10 mA
errorbar(x_values_block, mean(functional_blocks), std(functional_blocks), 'b', 'LineWidth', 2);
box on; xlabel('current [mA]'); ylabel('power [mW]'); title('LED blocks'); grid on;
xlim([min(x_values_block) max(x_values_block)]); ylim([0 2/1000])
text(0.5, 0.00175, ['n = ' num2str(size(functional_blocks, 1)) '/' num2str(size(block_leds, 1)) ' blocks'])

subplot(3, 1, 3); hold on;
x_values_all = driving_current_perc./100*max_current_all_uLEDs;
plot(x_values_all, all_leds, 'c'); errorbar(x_values_all, mean(all_leds), std(all_leds), 'b', 'LineWidth', 2);  % x-axis: maximum current used was 4*10 = 40 mA
box on; xlabel('current [mA]'); ylabel('power [mW]'); title('all LEDs'); grid on;
xlim([min(x_values_all) max(x_values_all)]); ylim([0 5/1000])
text(2, 0.0045, ['n = ' num2str(size(all_leds, 1))  ' oCIs'])


% intensity_estimates_individual = interp1(driving_current_perc,mean(functional_leds),0:10:100); intensity_estimates_y*1000;
% intensity_estimates_all = interp1(driving_current_perc,mean(all_leds),0:5:100); intensity_estimates_y*1000;
% StimCur_threshold = interp1(mean(all_leds.*1000), driving_current_perc/10*4, 0.35); % estimate threshold of current needed based on the light threshold from physiology experiments


