% this script outputs values and figures of the tonotopic slopes and their distribution recorded in each animal 
% without pharmacological deafening.  It also saves these slopes, which will later be used to estimate the spread 
% of excitation upon optogenetic stimulation.


%% prepare workspace
clear all;  clc; close all;

%% chose files to analyze and load data (CF vs depth), initialize variables
dirName = 'E:\projects\paper_uLED\revision\data\data_IC_MUA_acoustic\hearing\';
files_to_analyze = dir([dirName '\*mat']);
all_CFs = []; all_depths = []; all_IDs = {};

% collect depth, CFs and IDs for all animals in the experiment
for ii = 1:size(files_to_analyze, 1)
    
    cur_file = files_to_analyze(ii).name; load([dirName cur_file])
    
    data_all.(cur_file(1:end-10)) = data;
    all_IDs = vertcat(all_IDs, (cur_file(1:end-10)));
    
    all_CFs     = vertcat(all_CFs,      cell2mat(data((cell2mat(data(:, 2)) == 0), 3)));
    all_depths  = vertcat(all_depths,   cell2mat(data((cell2mat(data(:, 2)) == 0), 5)));    
    clear cur_file
    
end

% clean up CFs of zero/NaN (i.e. non-responsive electrodes/MUAs, as these would mess up the correlation)
all_CFs(all_CFs == 0) = NaN;
all_depths =  all_depths(~isnan(all_CFs));
all_CFs    =  all_CFs(~isnan(all_CFs)); all_CFs = all_CFs./1000; % convert from Hz into kHz

%% calculate and plot tonotopic slopes for each animal individually

f1 = figure('color', [1 1 1], 'units','normalized','position',[.3 .1 .25 .5]);
plotcols = [166,206,227;31,120,180;178,223,138;51,160,44;251,154,153;227,26,28;253,191,111;255,127,0;202,178,214;106,61,154;255,255,153;177,89,40];

slopes = [];
all_slopes_by_ID = {};

for ii = 1:length(all_IDs)
        
    % read in data
    curID = all_IDs(ii);cur_data = data_all.(curID{1});
    cur_fqs = cell2mat(cur_data(:, 3));     cur_depths = cell2mat(cur_data(:, 5));     
    
    % clean up CFs of zero/NaN (i.e. non-responsive electrodes/MUAs, as these would mess up the correlation)
    cur_fqs(cur_fqs == 0) = NaN; cur_depths =  cur_depths(~isnan(cur_fqs)); cur_fqs    =  cur_fqs(~isnan(cur_fqs)); 
    
    % convert from Hz into kHz
    cur_fqs = cur_fqs./1000;
    
    % calculate slope
    p = polyfit(log2(cur_fqs./0.5), cur_depths, 1);
    x = sort(log2(cur_fqs./0.5)); y = (p(1).*x)+p(2);
    
    % add some minor jitter on individual CFs to avoid overlay of data points in the plot
    oct_plus_noise =  log2(cur_fqs./500)+(0.05*randn(size(cur_fqs))); CFs_plus_noise = 500.*2.^oct_plus_noise;
    
    % plot the data of each animal with a given color 
    cur_plotcol = plotcols(ii, :);
    semilogx(sort(cur_fqs), y, 'k--', 'color', cur_plotcol./255, 'Linewidth', 0.5); hold on;
    semilogx(CFs_plus_noise, cur_depths, 'k.', 'color', cur_plotcol./255, 'MarkerSize', 12); hold on;
    
    % write the slopes of all animals in a common variable
    slopes = vertcat(slopes, p(1));
    all_slopes_by_ID{ii, 1} = curID;
    all_slopes_by_ID{ii, 2} = p(1);
    
end


%% linear regression for all data combined, include some parameters in plot and make plot look nicer
p = polyfit(log2(all_CFs./0.5), all_depths, 1);
x = sort(log2(all_CFs./0.5)); y = (p(1).*x)+p(2);
[R, P] = corrcoef([log2(all_CFs./0.5), all_depths]);

text(12, 1800, {['p = ' num2str(P(2, 1))]}, 'Fontsize', 9)
text(0.4, max(all_depths)+100, {['n = ' num2str(size(all_CFs, 1)) '/'  num2str(size(files_to_analyze, 1))]}, 'Fontsize', 9)
ylabel('depth [\mum]'); xlabel('CF [kHz]'); set(gca, 'XTick', [logspace(log10(0.25),log10(64),9)])
ylim([min(all_depths)-200 max(all_depths)+200]); xlim([0.3 50]); set(gca,'Ydir','reverse');


%% create and save output variable
data_tonotopy.all_slopes_by_ID = all_slopes_by_ID;
data_tonotopy.all_cfs_by_ID = data_all;
save (['all_slopes_by_ID.mat'], 'data_tonotopy');

% calculate median slope
octaves_per_mm = 1./ slopes.*1000; disp(['median slope = ' num2str(median(octaves_per_mm)) ' octaves/mm']);


%% plot distribution of tonotopic slopes
figure; title('tonotopic slopes')
hist(octaves_per_mm, [0:0.5:10]); xlim([floor(min(octaves_per_mm)) ceil(max(octaves_per_mm))-0.5]); ylim([0 15])
set(gca, 'XTick', [1:0.5:10]); xlabel('octaves/mm'); ylabel('number of animals');
ylim([0 4.5]); xlim([floor(min(octaves_per_mm)) ceil(max(octaves_per_mm))])
    
    

