% This script analyzes the tonotopic activation (i.e. best electrode as a function of emitter location), 
% and calculates potential correlations. The script contains some hard coded variables which describe the implant pitch of the oCIs 
% used in the corresponding datasets, which should not be changed. 
% Results are saved in a .mat-file. 

clear all; close all; clc;

%% read in data
path_base = 'E:\projects\paper_uLED\revision\analysis_final\';
uLEDs_block = load([path_base, '03c_STC_block_LEDs\STCs_uLED_blockwise_3to20ms_all']); % output from 03c_STC_block_LEDs

% general parameters
implant_pitch = [250 100 250 100 250 250 100 100 250 100 100 250 100 250 100 250 100 150 100 150 100 100 150 150 250 100 250 150 100 100 250 250];  
% The implant pitch in the above line describes the center to center pitch of LEDs on the differnt implants used in this study.
% the order of this corresponds to the oCIs which were used for the experiment, which are coded in this order in the script
% "RUN_STC_uLED_block_final" in folder "03c_STC_block_LEDs", which results in the data "STCs_uLED_blockwise_3to20ms_all"
% if this order will be changed, the data cannot be interpreted any longer in the right way.

electrode_spacing = 50; % in micrometer
n_LED_blocks = 4;

Dprime_threshold = 1.5; % to only include STCs with robust responses

um_per_octave_IC = 230.2; % from tonotopy analysis, median slope = 4.34 octaves/mm = 230.2 m per octave
um_per_octave_cochlea = 1380; % from literature reference

BEs = []; stimulus = []; exp_idx = []; pitch_idx = []; 

for n_exp = 1:size(uLEDs_block.analysis_part_02_metadata.dprime_cumsum, 3)/n_LED_blocks % for each stimulus block of 4, i.e. each implant...
  
    cur_thresholds = uLEDs_block.analysis_part_02_metadata.thresholds(n_exp*n_LED_blocks-n_LED_blocks+1:n_exp*n_LED_blocks, :); % ...load the thresholds
    cur_STCs = uLEDs_block.analysis_part_02_metadata.dprime_cumsum(:, :, n_exp*n_LED_blocks-n_LED_blocks+1:n_exp*n_LED_blocks); % ...and the spatial tuning curves
   
    if sum(max(max(cur_STCs)) > 2) > Dprime_threshold % if more than one block elicited responses above threshold  (to only include data with multiple evoked STCs)
        for i_stimulus = 1:n_LED_blocks % for each stimulus   
            if sum(~isnan(cur_thresholds(i_stimulus, :))) > 2 && max(max(cur_STCs(:, :, i_stimulus))) > 1.5 % if a d' of 1 was reached at 3 or more electrodes and the maximum response was higher as d' = 1.5 (to only include robust responses)
                
                [~, BE] = min(cur_thresholds(i_stimulus, :)); BEs = vertcat(BEs, BE);   % get the threshold 
                stimulus = vertcat(stimulus, i_stimulus);                               % assign the corresponding block
                exp_idx = vertcat(exp_idx, n_exp);                                      % assign the number of the experiment 
                pitch_idx = vertcat(pitch_idx, implant_pitch(n_exp));                   % assign the pitch of LEDs at this implant
           
            end
        end   
    end
end



experiments = unique(exp_idx); % identify all successful experiments
all_corrected_BEs = []; all_corrected_stimsites = [];

for i_exp = 1:length(experiments) % for each experiment...
    
    cur_exp = experiments(i_exp);
    
    cur_BEs = BEs(exp_idx == cur_exp);                  % load BEs
    cur_stimuli = stimulus(exp_idx == cur_exp);         % load the block of LED stimulatoin
    
    
    
    cur_pitch = unique(pitch_idx(exp_idx == cur_exp));  % load the pitch of the implant used
    cur_stimuli = cur_stimuli.*cur_pitch*4-cur_pitch;   % calculate the stimulus locatoin
               
    all_corrected_BEs       = vertcat(all_corrected_BEs, (cur_BEs-cur_BEs(1))); % correct BEs to the first BE
    all_corrected_stimsites  = vertcat(all_corrected_stimsites, (cur_stimuli-cur_stimuli(1)));  % correct stimulus locations to the first stimulus location
    
end


 
%% plot results
% define some markers for plotting and plot data
markercode = {'bo', 'bs', 'bd', 'b^', 'bv', 'b<', 'b>', 'b*', 'go', 'gs', 'gd', 'g^', 'gv', 'g<', 'g>', 'g*', 'ro', 'rs', 'rd', 'r^', 'rv', 'r<', 'r>', 'r*', 'mo', 'ms', 'md', 'm^', 'mv', 'm<', 'm>', 'm*'};
 
f1 = figure('color', [1 1 1], 'Position', [400 100 300 400]); 

for i_plot = 1:length(experiments)
    cur_exp = experiments(i_plot);
    plot(all_corrected_stimsites(exp_idx == cur_exp), all_corrected_BEs(exp_idx == cur_exp), markercode{i_plot}); hold on;
end

% adjust figure settings
set(gca,'Ydir','reverse'); 
plot([min(all_corrected_stimsites)-500 max(all_corrected_stimsites)+500], [0 0], 'r'); 
xlim([min(all_corrected_stimsites)-250 max(all_corrected_stimsites)+250]); 
plot([0 0], [min(all_corrected_BEs)-2 max(all_corrected_BEs)+2], 'r'); 
ylim([min(all_corrected_BEs)-1 max(all_corrected_BEs)+1]); 
grid on;
ylabel('relative BE'); xlabel('relative stimulus location [\mum]')


% calculate and display correlation between stimulus sites and BEs
[R, P] = corrcoef([all_corrected_stimsites, all_corrected_BEs]);
text(1500, 5.5, ['r = ' num2str(R(2))])
text(1500, 6, ['p = ' num2str(P(2))])

% plot linear fit of BEs as a function of sitmulus site
p = polyfit(all_corrected_stimsites, all_corrected_BEs, 1); x = sort(all_corrected_stimsites); y = (p(1).*x)+p(2); plot(x, y, 'k--')   

% as a reference, plot the tonotopy based on acoustic stimulation 
plot(3*um_per_octave_cochlea*[-1, 1], 3*[-1,1]*um_per_octave_IC./electrode_spacing , 'b-') % correct for electrode spacing

%% save data
data_block.stimsites = all_corrected_stimsites;
data_block.BEs = all_corrected_BEs;
save('tonotopy_block', 'data_block')

