% This script analyzes the spectral spread, i.e. the space covered by responding electrodes, at desired d values at the best electrode. 
% It also saves intermediate results for visual inspection (even though the figure quality is quite poor, and always scaled to the full range of stimuli, so one might have to zoom in). 
% It saves the spatial spread sorted by LED-pitch of the implants as well as the IDs of the animal that this data was recorded from, 
% so it can be converted into spectral spread by using the tonotopic slopes calculated earlier. 

%% clear workspace, load data
clear all; close all; clc;

path_base = 'E:\projects\paper_uLED\revision\analysis_final\';
uLEDs_block = load([path_base, '03c_STC_block_LEDs\STCs_uLED_blockwise_3to20ms_all']); % load in data (output of RUNanalysis_STC_uLED_block)

% general parameters
implant_pitch = [  250 250 250 250 100 100 100 100 250 250 250 250 100 100 100 100 250 250 250 250 250 250 250 250 100 100 100 100 100 100 100 100 ...
    250 250 250 250 100 100 100 100 100 100 100 100 250 250 250 250 100 100 100 100 250 250 250 250 100 100 100 100 250 250 250 250 ...
    100 100 100 100 150 150 150 150 100 100 100 100 150 150 150 150 100 100 100 100 100 100 100 100 150 150 150 150 100 100 100 100 ...
    250 250 250 250 100 100 100 100 250 250 250 250 150 150 150 150 100 100 100 100 100 100 100 100 250 250 250 250 250 250 250 250];  % this vector contains the center-to-center pitch of different LED implants used for stimulation. The order corresponds to the data wich is loaded in the line before. DO NOT CHANGE THIS!!!
% The implant pitch in the above line describes the center to center pitch of LEDs on the differnt implants used in this study.
% the order of this corresponds to the oCIs which were used for the experiment, which are coded in this order in the script
% "RUN_STC_uLED_block_final" in folder "03c_STC_block_LEDs", which results in the data "STCs_uLED_blockwise_3to20ms_all"
% if this order will be changed, the data cannot be interpreted any longer in the right way.

dprime_values = [1.5 2 2.5 3]; % define the d' values at which the spread of excitation should be analyzed (above 1, as 1 is considered the threshold)
DprimeThreshold  = 2; % only STCs with a maximum response above this threshold are included in the analysis in order to only include robust repsonses

n_electrodes = 32;
n_LED_blocks = 4;

electrode_spacing = 50; % in micrometers

% reference data from Dieter et al, Nat Comm 2019
bw_acoustic = [184 293 356 455]; % in micrometers
bw_opto_fiber =  [214  394  610  792]; 
bw_eStim_mono = [411 975 1365 1453];

%% main analysis
STC_idx = [];  all_STCs = []; bandwidths = []; dprime_threshold = [];% initiate variables
for n_STC = 1:size(uLEDs_block.analysis_part_02_metadata.dprime_cumsum, 3) % for each STC
    
    curID = uLEDs_block.analysis_part_02_metadata.IDs(floor((n_STC-1)/n_LED_blocks)+1); % get the current animal ID
    cur_thresholds = uLEDs_block.analysis_part_02_metadata.thresholds(n_STC, :); % get current thresholds
    cur_STC = uLEDs_block.analysis_part_02_metadata.dprime_cumsum(:, :, n_STC); % get current STCs
    
    if  max(max(cur_STC)) > DprimeThreshold  % if a d-prime of at least 2 was elicited (to only include robust repsonses)
        STC_idx = vertcat(STC_idx, n_STC); % add an identifier for each analyzed STC
        
        for idx_dprime = 1:length(dprime_values) % for all chosen d-prime values
            
            dprime  = dprime_values(idx_dprime); % get the current d prime value of interest
            
            clear contour_matrix1 contour_matrix2 contour_line1 contour_line2  lower_electrode_interp  upper_electrode_interp upper_electrode_off_band lower_electrode_off_band dprime_threshold dprime2test %  clear variables to determine bandwith of excitation
            channels = [1:n_electrodes]'; threshold_by_channel = zeros(1, n_electrodes)'; dprime2test_by_channel = zeros(1, n_electrodes)';
            minval_threshold = NaN; minval_dprime2test = NaN;
            
            f0 = figure;
            try [contour_matrix1, ~] = contourf(cur_STC,[1 1]); contour_line1 = getcontourlines(contour_matrix1); end; hold on; % extract contour line for threshold
            try [contour_matrix2, ~] = contourf(cur_STC, [dprime dprime]); contour_line2 = getcontourlines(contour_matrix2); end % extract contour line for desired d' value
            close (f0)
            
            f1 = figure('color', [1 1 1], 'units','normalized','position',[.1 .1 0.35 .7]); hold on;
            subplot(1, 2, 1); hold on; contourf(cur_STC, [ 1:0.5:ceil(max(max(cur_STC)))]); ylim([1 n_electrodes]); xlim([0 11]); % plot current STC at a spacing of 0.5 d' values above threshold
            
            if exist('contour_line2') % if a contour was found for the d' value at which the bandwidth should be determined (i.e. if the STC contains this value)
                
                % determine the coordinates of the threshold
                i_dprime_threshold  = 0;
                for zz = 1:size(contour_line1, 2)
                    if contour_line1(zz).x(1) == contour_line1(zz).x(end) && contour_line1(zz).y(1) == contour_line1(zz).y(end) % contour is closed, ignore it
                    else i_dprime_threshold = i_dprime_threshold + 1; % if contour is open (i.e. the strongest stimuli elicit responses), consider them as threshold
                        dprime_threshold(i_dprime_threshold).x = contour_line1(zz).x; dprime_threshold(i_dprime_threshold).y = contour_line1(zz).y;
                    end
                end
                
                % determine the threshold per channel
                for yy = 1:size(dprime_threshold, 2) % for all detected thresholds (there might be two "responsive areas" in a STC...
                    cur_thresholds = dprime_threshold(yy);
                    for cur_channel = 1:size(channels) % ...and for each channel...
                        if nnz(ismember(cur_thresholds.y, cur_channel)) > 0 % ...check if a threshold was detected for this channel
                            idx_threshold = find(ismember(cur_thresholds.y, cur_channel));
                            cur_threshold = cur_thresholds.x(idx_threshold); % read out the threshold
                            if length(cur_threshold) > 1
                                [minval, minloc] = min(cur_threshold);
                                cur_threshold = cur_threshold(minloc);
                                threshold_by_channel(cur_channel) = cur_threshold;
                            else
                                threshold_by_channel(cur_channel) = cur_threshold;
                            end
                        end
                    end
                end
                
                threshold_by_channel(threshold_by_channel == 0) = size(cur_STC, 2); % assign the highest stimulus intensity tested to the electrodes which do not respond
                
                % determine the coordinates of the d' value to be tested (identical procedure as the two loops above)
                i_dprime2test  = 0;
                for zz = 1:size(contour_line2, 2)
                    if contour_line2(zz).x(1) == contour_line2(zz).x(end) && contour_line2(zz).y(1) == contour_line2(zz).y(end) % contour is closed, ignore it
                    else  i_dprime2test = i_dprime2test + 1; % if contour is open (i.e. the strongest stimuli elicit responses), consider them as threshold
                        dprime2test(i_dprime2test).x = contour_line2(zz).x; dprime2test(i_dprime2test).y = contour_line2(zz).y;
                    end
                end
                
                if exist('dprime2test','var') % if outlines were found...
                    
                    for yy = 1:size(dprime2test, 2) % for all detected thresholds (there might be two "responsive areas" in a STC...
                        cur_thresholds = dprime2test(yy);
                        for cur_channel = 1:size(channels) % ...and for each channel...
                            if nnz(ismember(cur_thresholds.y, cur_channel)) > 0 % ...check if a threshold was detected for this channel
                                idx_threshold = find(ismember(cur_thresholds.y, cur_channel));
                                cur_threshold = cur_thresholds.x(idx_threshold); % read out the threshold
                                
                                if length(cur_threshold) > 1
                                    [minval, minloc] = min(cur_threshold);
                                    cur_threshold = cur_threshold(minloc);
                                    dprime2test_by_channel(cur_channel) = cur_threshold;
                                else
                                    dprime2test_by_channel(cur_channel) = cur_threshold;
                                end
                            end
                        end
                    end
                    
                    dprime2test_by_channel(dprime2test_by_channel == 0) = size(cur_STC, 2); % assign the highest stimulus intensity tested to the electrodes which do not respond
                    
                    % plot the determined threshold and outlines of d' vlaue to be tested
                    subplot(1, 2, 2); hold on;
                    plot(threshold_by_channel, channels, 'g*--'); plot(dprime2test_by_channel, channels, 'r*--');
                    ylim([1 n_electrodes]); xlim([0 11])
                    
                    [minval_threshold, minloc_threshold] = min(threshold_by_channel); % determine best electrode
                    plot(minval_threshold, minloc_threshold, 'go');
                    minval_dprime2test = dprime2test_by_channel(minloc_threshold); % determine minimum intensity at which the d' to be tested is eached at the best electrode
                    
                else % if no outlines were found for the d' vlaue to be tested, ignore this STC
                    text(3, 16, 'no d-prime > 2 for best electrode');
                    clear  dprime_threshold dprime2test
                end
                
                if minval_dprime2test < size(cur_STC, 2) % as the highest intensity was assigned to electrodes with no response, the d-prime must be smaller than this, in order to indicate a responsive STC
                    
                    plot(minval_dprime2test, minloc_threshold, 'ro'); plot([minval_dprime2test minval_dprime2test], [1 n_electrodes], 'k--')  % plot the d' value to be tested
                    
                    resp_electrodes_at_dprime2test = find(threshold_by_channel < minval_dprime2test); % determine responsive electrodes (i.e. d' >= 1) at the intensity that evokes the d' to be tested at the BE
                    dorsal_electrode_in = min(resp_electrodes_at_dprime2test); dorsal_electrode_out = dorsal_electrode_in-1; % determine the dorsal-most electrodes within and outside the responding band of electrodes
                    ventral_electrode_in = max(resp_electrodes_at_dprime2test); ventral_electrode_out = ventral_electrode_in+1; % determine the ventral-most electrodes within and outside the responding band of electrodes
                    
                    % actually calculate bandwidth
                    if ventral_electrode_out <= n_electrodes && dorsal_electrode_out >= 1 % if the dorsal and the ventral edge of the STC is covered by the electrode array
                        
                        % interpolate the edge of the spatial tuning curve
                        dorsal_electrode_interp = interp1([threshold_by_channel(dorsal_electrode_out) threshold_by_channel(dorsal_electrode_in)],[dorsal_electrode_out dorsal_electrode_in],minval_dprime2test);
                        ventral_electrode_interp = interp1([threshold_by_channel(ventral_electrode_out) threshold_by_channel(ventral_electrode_in)],[ventral_electrode_out ventral_electrode_in],minval_dprime2test);
                        
                        % plot responsive electrodes, doral and ventral boundaries for visual control
                        plot([minval_dprime2test threshold_by_channel(dorsal_electrode_in:ventral_electrode_in)' minval_dprime2test], [dorsal_electrode_interp dorsal_electrode_in:ventral_electrode_in ventral_electrode_interp], 'b-', 'LineWidth', 2);
                        plot(minval_dprime2test,dorsal_electrode_interp , 'b*'); plot(minval_dprime2test,ventral_electrode_interp , 'b*');
                        
                        % calculate and plot bandwidth based on the electrode spacing (i.e. 50 m) for visual control
                        bandwidth = (ventral_electrode_interp-dorsal_electrode_interp).*electrode_spacing;
                        text(1, 31, ['bandwidth = ', num2str(round(bandwidth)) ' \mum']);
                        
                        % store bandwidth and the ID of the animal (to later relate it to the tonotopic slope)
                        bandwidths (length(STC_idx), idx_dprime) = bandwidth; bandwidth_IDs{n_STC} =  curID;
                        clear dprime_threshold dprime2test
                        
                    elseif ventral_electrode_out <= n_electrodes && dorsal_electrode_out < 1  % if the dorsal electrode is not covered by the electrode array
                        % interpolate the ventral edge of the spatial tuning curve and assign the  most dorsal electrodes as boundaries
                        ventral_electrode_interp = interp1([threshold_by_channel(ventral_electrode_out) threshold_by_channel(ventral_electrode_in)],[ventral_electrode_out ventral_electrode_in],minval_dprime2test);
                        dorsal_electrode_out = 1;
                        
                        % plot responsive electrodes, doral and ventral boundaries for visual control
                        plot([minval_dprime2test threshold_by_channel(dorsal_electrode_in:ventral_electrode_in)' minval_dprime2test], [dorsal_electrode_out dorsal_electrode_in:ventral_electrode_in ventral_electrode_interp], 'b-', 'LineWidth', 2);
                        plot(minval_dprime2test,dorsal_electrode_out , 'b*'); plot(minval_dprime2test,ventral_electrode_interp , 'b*');
                        
                        bandwidth = (ventral_electrode_interp - dorsal_electrode_out).*electrode_spacing;
                        
                        % calculate and plot bandwidth based on the electrode spacing (i.e. 50 m) for visual control
                        bandwidth = (ventral_electrode_interp - dorsal_electrode_out).*electrode_spacing;
                        text(1, 31, ['bandwidth = ', num2str(round(bandwidth)) ' \mum']);
                        
                        % store bandwidth and the ID of the animal (to later relate it to the tonotopic slope)
                        bandwidths (length(STC_idx), idx_dprime) = bandwidth; bandwidth_IDs{n_STC} =  curID;
                        clear dprime_threshold dprime2test
                        
                    elseif ventral_electrode_out > n_electrodes && dorsal_electrode_out >= 1 % if the ventral electrode is not covered by the electrode array
                        % interpolate the dorsal edge of the spatial tuning curve and assign the  most ventral electrodes as boundaries
                        dorsal_electrode_interp = interp1([threshold_by_channel(dorsal_electrode_out) threshold_by_channel(dorsal_electrode_in)],[dorsal_electrode_out dorsal_electrode_in],minval_dprime2test);
                        ventral_electrode_out = n_electrodes;
                        
                        % plot responsive electrodes, doral and ventral boundaries for visual control
                        plot([minval_dprime2test threshold_by_channel(dorsal_electrode_in:ventral_electrode_in)' minval_dprime2test], [dorsal_electrode_interp dorsal_electrode_in:ventral_electrode_in ventral_electrode_out], 'b-', 'LineWidth', 2);
                        plot(minval_dprime2test,dorsal_electrode_interp , 'b*'); plot(minval_dprime2test,ventral_electrode_out , 'b*');
                        
                        % calculate and plot bandwidth based on the electrode spacing (i.e. 50 m) for visual control
                        bandwidth = (ventral_electrode_out - dorsal_electrode_interp).*electrode_spacing;
                        text(1, 31, ['bandwidth = ', num2str(round(bandwidth)) ' \mum']);
                        
                        % store bandwidth and the ID of the animal (to later relate it to the tonotopic slope)
                        bandwidths (length(STC_idx), idx_dprime) = bandwidth; bandwidth_IDs{n_STC} =  curID;
                        clear dprime_threshold dprime2test
                        
                    elseif ventral_electrode_out > n_electrodes && dorsal_electrode_out < 1 % if both the dorsal and the ventral edge of the STC is outside of the electrode array
                        % assign the most dorsal and most ventral electrodes as boundaries
                        ventral_electrode_out = n_electrodes;  dorsal_electrode_out = 1;
                        
                        % plot responsive electrodes, doral and ventral boundaries for visual control
                        plot([threshold_by_channel], [1:n_electrodes], 'b-', 'LineWidth', 2);
                        
                        % calculate and plot bandwidth based on the electrode spacing (i.e. 50 m) for visual control
                        bandwidth = (ventral_electrode_out-dorsal_electrode_out).*electrode_spacing;
                        text(1, 30, ['bandwidth = ', num2str(round(bandwidth)) ' \mum']);
                        text(1, 31, 'no limits detected');
                        
                        % store bandwidth and the ID of the animal (to later relate it to the tonotopic slope)
                        bandwidths (length(STC_idx), idx_dprime) = bandwidth; bandwidth_IDs{n_STC} =  curID;
                        clear dprime_threshold dprime2test
                    end
                end
                
                clear  dprime_threshold dprime2test contour_line1 contour_line2
                
            else % if no contour was found for the d' value at which the bandwidth should be determined, assign the bandwidth with NaN in this case
                bandwidths (length(STC_idx), idx_dprime) = NaN; bandwidth_IDs{n_STC} =  curID;
                subplot(1, 2, 2); hold on; text(3, 16, ['no d-prime = ' num2str(dprime) ' in this file'])
                ylim([1 n_electrodes]); xlim([0 11]);
                clear  dprime_threshold dprime2test contour_line1 contour_line2
            end
            
            % save the figure for visual inspection. This can be commented if you are low on memory
             try name2save = ['STC_' num2str(n_STC) '_' num2str(idx_dprime) '.png']; saveas(gcf,name2save); end
            
            close all; clear contour_matrix1 contour_matrix2  contour_line1 contour_line2  lower_electrode_interp  upper_electrode_interp upper_electrode_off_band lower_electrode_off_band dprime_threshold dprime2test %  clear variables to determine bandwith of excitation
            
        end
    end
end


bandwidths(bandwidths == 0) = NaN;

% assign a index to group implants with a pitch of 100/150 m or 250 m
idx_pitch_100 = implant_pitch(STC_idx)' == 100 | implant_pitch(STC_idx)' == 150; idx_pitch_250 = implant_pitch(STC_idx)' == 250;
% sort the calculated bandwidths based on the implant they were evoked with
bandwidths_100 = bandwidths(idx_pitch_100, :); bandwidths_250 = bandwidths(idx_pitch_250, :);


% get rid of empty cells in the array which correspond to the ID of STCs that have not been analyzed 
ID_pos = 1;
for iii = 1:size(bandwidth_IDs, 2)
    if ~isempty(bandwidth_IDs{iii})
         band_width_IDs_new{ID_pos} = bandwidth_IDs{iii};
         ID_pos = ID_pos + 1;
    end
end

bandwidth_IDs_100 = band_width_IDs_new(idx_pitch_100); bandwidth_IDs_250 = band_width_IDs_new(idx_pitch_250);





%% plot data
f1 = figure('color', [1 1 1]);  hold on;

plot(dprime_values(1)-0.1:0.2./(size(bandwidths_100, 1)-1):dprime_values(1)+0.1, bandwidths_100(:, 1), 'bo', 'color', [67 162 202]./255, 'MarkerFaceColor', [67 162 202]./255)
plot(dprime_values(2)-0.1:0.2./(size(bandwidths_100, 1)-1):dprime_values(2)+0.1, bandwidths_100(:, 2), 'bo', 'color', [67 162 202]./255,'MarkerFaceColor', [67 162 202]./255)
plot(dprime_values(3)-0.1:0.2./(size(bandwidths_100, 1)-1):dprime_values(3)+0.1, bandwidths_100(:, 3), 'bo', 'color', [67 162 202]./255,'MarkerFaceColor', [67 162 202]./255)
plot(dprime_values(4)-0.1:0.2./(size(bandwidths_100, 1)-1):dprime_values(4)+0.1, bandwidths_100(:, 4), 'bo', 'color', [67 162 202]./255,'MarkerFaceColor', [67 162 202]./255)

plot(dprime_values(1)-0.1:0.2./(size(bandwidths_250, 1)-1):dprime_values(1)+0.1, bandwidths_250(:, 1), 'bo', 'MarkerFaceColor', 'b')
plot(dprime_values(2)-0.1:0.2./(size(bandwidths_250, 1)-1):dprime_values(2)+0.1, bandwidths_250(:, 2), 'bo', 'MarkerFaceColor', 'b')
plot(dprime_values(3)-0.1:0.2./(size(bandwidths_250, 1)-1):dprime_values(3)+0.1, bandwidths_250(:, 3), 'bo', 'MarkerFaceColor', 'b')
plot(dprime_values(4)-0.1:0.2./(size(bandwidths_250, 1)-1):dprime_values(4)+0.1, bandwidths_250(:, 4), 'bo', 'MarkerFaceColor', 'b')


errorbar(dprime_values(1), nanmean( bandwidths_100(:, 1)), nanstd( bandwidths_100(:, 1))./sqrt(length(bandwidths_100(:, 1))), 'color', [67 162 202]./255, 'LineWidth', 2)
errorbar(dprime_values(2), nanmean( bandwidths_100(:, 2)), nanstd( bandwidths_100(:, 2))./sqrt(length(bandwidths_100(:, 2))), 'color', [67 162 202]./255, 'LineWidth', 2)
errorbar(dprime_values(3), nanmean( bandwidths_100(:, 3)), nanstd( bandwidths_100(:, 3))./sqrt(length(bandwidths_100(:, 3))), 'color', [67 162 202]./255, 'LineWidth', 2)
errorbar(dprime_values(4), nanmean( bandwidths_100(:, 4)), nanstd( bandwidths_100(:, 4))./sqrt(length(bandwidths_100(:, 4))), 'color', [67 162 202]./255, 'LineWidth', 2)
plot(dprime_values, nanmean(bandwidths_100), 'color', [67 162 202]./255, 'LineWidth', 2)

errorbar(dprime_values(1), nanmean( bandwidths_250(:, 1)), nanstd( bandwidths_250(:, 1))./sqrt(length(bandwidths_250(:, 1))), 'b', 'LineWidth', 2)
errorbar(dprime_values(2), nanmean( bandwidths_250(:, 2)), nanstd( bandwidths_250(:, 2))./sqrt(length(bandwidths_250(:, 2))), 'b', 'LineWidth', 2)
errorbar(dprime_values(3), nanmean( bandwidths_250(:, 3)), nanstd( bandwidths_250(:, 3))./sqrt(length(bandwidths_250(:, 3))), 'b', 'LineWidth', 2)
errorbar(dprime_values(4), nanmean( bandwidths_250(:, 4)), nanstd( bandwidths_250(:, 4))./sqrt(length(bandwidths_250(:, 4))), 'b', 'LineWidth', 2)
plot(dprime_values, nanmean(bandwidths_250), 'b', 'LineWidth', 2)

xlim([1 3.5]); ylim([0 1600]); 

ylabel('spatial spread [\mum]'); xlabel('d-prime')


% plot some reference data from Dieter et al, Nat Comm 2019
plot(dprime_values, bw_acoustic, 'k--', 'LineWidth', 2);
plot(dprime_values, bw_opto_fiber, 'b--', 'LineWidth', 2);
plot(dprime_values, bw_eStim_mono, 'r--', 'LineWidth', 2);


%% save the data
data.dprime_values = dprime_values; 
data.band_widths_100 = bandwidths_100;
data.band_widths_250 = bandwidths_250;
data.band_width_IDs_100 = bandwidth_IDs_100;
data.band_width_IDs_250 = bandwidth_IDs_250;

save (['spectral_spread_uLED_block.mat'], 'data')

